"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HomeWizardEnergyDiscovery = void 0;
const multicastDns = __importStar(require("multicast-dns"));
const types_1 = require("./types");
const buffer_1 = require("./utils/buffer");
const util_1 = __importDefault(require("util"));
/**
 * HomeWizard Energy Discovery
 *
 * @link: https://homewizard-energy-api.readthedocs.io/discovery.html
 */
class HomeWizardEnergyDiscovery {
    constructor(options) {
        this.mdns = null;
        this.cachedDiscoveryResponses = [];
        this.loggerOptions = options?.logger;
    }
    log(...args) {
        if (!this.loggerOptions?.method)
            return;
        const loggerPrefix = this.loggerOptions.prefix || '[HomeWizard Energy API]:';
        return this.loggerOptions.method(loggerPrefix, ...args);
    }
    start() {
        this.log('start');
        this.mdns = multicastDns.default();
        this.mdns.query(types_1.MDNS_DISCOVERY_DOMAIN, types_1.MDNS_DISCOVERY_QUERY_TYPE);
    }
    stop() {
        this.log('stop');
        if (this.mdns) {
            this.mdns.removeAllListeners();
            this.mdns.destroy();
            this.mdns = null;
        }
    }
    on(event, 
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    callback) {
        if (!this.mdns) {
            throw new Error('mDNS is not started, call discovery.start() first');
        }
        // if (event === 'down') {
        //   this.on(event, this.handleMdnsDown(callback));
        //   return;
        // }
        if (event === 'error') {
            this.mdns.on(event, this.handleMdnsError(callback));
            return;
        }
        if (event === 'warning') {
            this.mdns.on(event, this.handleMdnsWarning(callback));
            return;
        }
        this.mdns.on(event, this.handleMdnsResponse(callback));
    }
    // TODO: make this handler work
    // protected handleMdnsDown(callback: (goodbyeResponse: DiscoveryResponse) => void) {
    //   return (response: multicastDns.ResponsePacket) => {
    //     console.log('goodbye!', response);
    //     // TODO: fix
    //     callback(response as unknown as DiscoveryResponse);
    //   };
    // }
    isGoodbyeResponse(response) {
        return response.answers.some(answer => {
            if ('ttl' in answer) {
                return (answer.type === types_1.MDNS_DISCOVERY_QUERY_TYPE &&
                    answer.ttl === 0 &&
                    answer.data.includes(types_1.MDNS_DISCOVERY_TYPE));
            }
            return false;
        });
    }
    /**
     * Find out if the response is a HomeWizard Energy response.
     */
    isHomeWizardEnergyResponse(response) {
        return (response.answers?.filter(answer => {
            if ('data' in answer && typeof answer.data === 'string') {
                return answer.data.includes(types_1.MDNS_DISCOVERY_TYPE);
            }
            return false;
        }).length > 0);
    }
    getTxtRecordFromResponse(response) {
        const txt = response.answers.find(answer => answer.type === 'TXT');
        // no txt record found
        if (!txt)
            return null;
        // no data property found
        if (!('data' in txt))
            return null;
        // data is not an array of buffers
        if (!(0, buffer_1.isBufferArray)(txt.data))
            return null;
        // txt.data must be an Array of Buffers when we end up here
        const txtRecordFromBuffers = (0, buffer_1.bufferArrayToJSON)(txt.data);
        return txtRecordFromBuffers;
    }
    getHostDetailsFromAnswer(answer) {
        if (!('data' in answer))
            return null;
        if (typeof answer.data !== 'string')
            return null;
        return { ip: answer.data, hostname: answer.name };
    }
    getHostDetailsFromResponse(response) {
        // We can find the host details in the answer or additional section
        const answer = response.answers.find(answer => answer.type === 'A');
        const additional = response.additionals.find(additional => additional.type === 'A');
        if (!answer && !additional)
            return null;
        if (answer && !additional) {
            return this.getHostDetailsFromAnswer(answer);
        }
        if (additional) {
            return this.getHostDetailsFromAnswer(additional);
        }
        return null;
    }
    getFqdnFromResponse(response) {
        const ptr = response.answers.find(a => a.type === 'PTR' && a.name?.includes(types_1.MDNS_DISCOVERY_TYPE));
        if (!ptr) {
            return null;
        }
        if (ptr && !('data' in ptr)) {
            return null;
        }
        if (typeof ptr.data !== 'string') {
            return null;
        }
        return ptr.data;
    }
    getCachedResponseByFqdn(fqdn) {
        return this.cachedDiscoveryResponses.find(cachedResponse => cachedResponse.fqdn === fqdn);
    }
    removeCachedResponseByFqdn(fqdn) {
        this.cachedDiscoveryResponses = this.cachedDiscoveryResponses.filter(cachedResponse => cachedResponse.fqdn !== fqdn);
    }
    isDiscoveryResponseInCache(discoveryResponse) {
        return this.cachedDiscoveryResponses.some(cachedDiscoveryResponse => util_1.default.isDeepStrictEqual(discoveryResponse, cachedDiscoveryResponse));
    }
    handleMdnsResponse(callback) {
        return (response) => {
            const isHomeWizardEnergyResponse = this.isHomeWizardEnergyResponse(response);
            this.log(`Received response from mDNS: ${JSON.stringify(response)}`);
            if (!isHomeWizardEnergyResponse) {
                this.log('Response from mDNS is not from HomeWizard Energy, ignoring.');
                return;
            }
            // Get the FQDN from the response, we use this as a unique identifier for the device
            const fqdn = this.getFqdnFromResponse(response);
            if (!fqdn) {
                this.log(`No fqdn found in mDNS response, ignoring: ${JSON.stringify(response)}`);
                return;
            }
            if (this.isGoodbyeResponse(response)) {
                this.log(`Response is a goodbye message, we'll send a down event.`);
                const cachedResponse = this.getCachedResponseByFqdn(fqdn);
                if (!cachedResponse) {
                    this.log(`No cached response found for ${fqdn}, ignoring.`);
                }
                this.log('down!', cachedResponse);
                // this.emit('down', cachedResponse);
                // Remove the response from cache by using the fqdn, which is specific enough
                this.removeCachedResponseByFqdn(fqdn);
                return;
            }
            // Response is from hwenergy
            const hostDetails = this.getHostDetailsFromResponse(response);
            const txtRecord = this.getTxtRecordFromResponse(response);
            // This should not happen, but just in case
            if (!hostDetails) {
                throw new Error(`No host details found in mDNS response: ${JSON.stringify(response)}`);
            }
            // This should not happen, but just in case
            if (!txtRecord) {
                throw new Error(`No txt record data found in mDNS response: ${JSON.stringify(response)}`);
            }
            const discoveryResponse = {
                ip: hostDetails.ip,
                hostname: hostDetails.hostname,
                fqdn,
                txt: txtRecord,
            };
            const isDiscoveryResponseInCache = this.isDiscoveryResponseInCache(discoveryResponse);
            if (isDiscoveryResponseInCache) {
                this.log('Nothing changed. Response is already in cache, ignoring.');
                return;
            }
            // Store response in cache, so we don't send multiple events for the same device when not needed
            this.cachedDiscoveryResponses.push(discoveryResponse);
            return callback(discoveryResponse);
        };
    }
    handleMdnsError(callback) {
        return (error) => {
            callback(error);
        };
    }
    handleMdnsWarning(callback) {
        return (error) => {
            callback(error);
        };
    }
}
exports.HomeWizardEnergyDiscovery = HomeWizardEnergyDiscovery;
//# sourceMappingURL=homewizard-energy-discovery.js.map